function [allData, scenario, sensor] = cmpxScnaraio()
%cmpxScnaraio - Returns sensor detections
%    allData = cmpxScnaraio returns sensor detections in a structure
%    with time for an internally defined scenario and sensor suite.
%
%    [allData, scenario, sensors] = cmpxScnaraio optionally returns
%    the drivingScenario and detection generator objects.

% Generated by MATLAB(R) 9.7 (R2019b) and Automated Driving Toolbox 3.0 (R2019b).
% Generated on: 23-Jul-2020 20:52:19

% Create the drivingScenario object and ego car
[scenario, egoVehicle] = createDrivingScenario;

% Create all the sensors
sensor = createSensor(scenario);

allData = struct('Time', {}, 'ActorPoses', {}, 'ObjectDetections', {}, 'LaneDetections', {});
running = true;
while running
    
    % Generate the target poses of all actors relative to the ego vehicle
    poses = targetPoses(egoVehicle);
    time  = scenario.SimulationTime;
    
    % Generate detections for the sensor
    laneDetections = [];
    [objectDetections, numObjects, isValidTime] = sensor(poses, time);
    objectDetections = objectDetections(1:numObjects);
    
    % Aggregate all detections into a structure for later use
    if isValidTime
        allData(end + 1) = struct( ...
            'Time',       scenario.SimulationTime, ...
            'ActorPoses', actorPoses(scenario), ...
            'ObjectDetections', {objectDetections}, ...
            'LaneDetections',   {laneDetections}); %#ok<AGROW>
    end
    
    % Advance the scenario one time step and exit the loop if the scenario is complete
    running = advance(scenario);
end

% Restart the driving scenario to return the actors to their initial positions.
restart(scenario);

% Release the sensor object so it can be used again.
release(sensor);

%%%%%%%%%%%%%%%%%%%%
% Helper functions %
%%%%%%%%%%%%%%%%%%%%

% Units used in createSensors and createDrivingScenario
% Distance/Position - meters
% Speed             - meters/second
% Angles            - degrees
% RCS Pattern       - dBsm

function sensor = createSensor(scenario)
% createSensors Returns all sensor objects to generate detections

% Assign into each sensor the physical and radar profiles for all actors
profiles = actorProfiles(scenario);
sensor = radarDetectionGenerator('SensorIndex', 1, ...
    'SensorLocation', [3.7 0], ...
    'MaxRange', 100, ...
    'ActorProfiles', profiles);

function [scenario, egoVehicle] = createDrivingScenario
% createDrivingScenario Returns the drivingScenario defined in the Designer

% Construct a drivingScenario object.
scenario = drivingScenario;

% Add all road segments
roadCenters = [0 1500 0;
    0 0 0];
marking = laneMarking('Dashed', 'Length', 1.5, 'Space', 3);
laneSpecification = lanespec(2, 'Width', 2.925, 'Marking', marking);
road(scenario, roadCenters, 'Lanes', laneSpecification);

roadCenters = [-500 500 0;
    500 500 0];
marking = laneMarking('Dashed', 'Length', 1.5, 'Space', 3);
laneSpecification = lanespec(2, 'Width', 2.925, 'Marking', marking);
road(scenario, roadCenters, 'Lanes', laneSpecification);

roadCenters = [500 1000 0;
    -500 1000 0];
marking = laneMarking('Dashed', 'Length', 1.5, 'Space', 3);
laneSpecification = lanespec(2, 'Width', 2.925, 'Marking', marking);
road(scenario, roadCenters, 'Lanes', laneSpecification);

roadCenters = [-500 500 0;
    500 500 0];
marking = laneMarking('Dashed', 'Length', 1.5, 'Space', 3);
laneSpecification = lanespec(2, 'Width', 2.925, 'Marking', marking);
road(scenario, roadCenters, 'Lanes', laneSpecification);

roadCenters = [-100 250 0;
    200 250 0];
marking = laneMarking('Dashed', 'Length', 1.5, 'Space', 3);
laneSpecification = lanespec(2, 'Width', 2.925, 'Marking', marking);
road(scenario, roadCenters, 'Lanes', laneSpecification);

roadCenters = [200 250 0;
    300 400 0];
marking = laneMarking('Dashed', 'Length', 1.5, 'Space', 3);
laneSpecification = lanespec(2, 'Width', 2.925, 'Marking', marking);
road(scenario, roadCenters, 'Lanes', laneSpecification);

roadCenters = [300 400 0;
    300 600 0];
marking = laneMarking('Dashed', 'Length', 1.5, 'Space', 3);
laneSpecification = lanespec(2, 'Width', 2.925, 'Marking', marking);
road(scenario, roadCenters, 'Lanes', laneSpecification);

roadCenters = [300 600 0;
    200 750 0];
marking = laneMarking('Dashed', 'Length', 1.5, 'Space', 3);
laneSpecification = lanespec(2, 'Width', 2.925, 'Marking', marking);
road(scenario, roadCenters, 'Lanes', laneSpecification);

roadCenters = [200 750 0;
    -100 750 0];
marking = laneMarking('Dashed', 'Length', 1.5, 'Space', 3);
laneSpecification = lanespec(2, 'Width', 2.925, 'Marking', marking);
road(scenario, roadCenters, 'Lanes', laneSpecification);

% Add the ego vehicle
egoVehicle = vehicle(scenario, ...
    'ClassID', 1, ...
    'Position', [1.5 1500 0], ...
    'FrontOverhang', 0.9);
waypoints = [1.5 1500 0;
    1.5 0 0];
speed = 30;
trajectory(egoVehicle, waypoints, speed);

% Add the non-ego actors
car1 = vehicle(scenario, ...
    'ClassID', 1, ...
    'Position', [-1.5 0 0], ...
    'FrontOverhang', 0.9);
waypoints = [-1.5 0 0;
    -1.5 1500 0];
speed = 30;
trajectory(car1, waypoints, speed);

