import sys
import gym
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from newddpg_PER import DDPGagent
from newutils import OUNoise, Randnormalnoise
import CoMoVe_RL_MultipleSteps_option2 as frmwrk
from tqdm import tqdm

if __name__ == "__main__":
    
    seed = 0
    np.random.seed(seed)
    
    env = frmwrk.SimulationEnvironment()
    
    batch_size = 32 #1; 2; 4; 8; 16; 32; 64; 128; 256
    droprob = 0 #0; 0.1; 0.2; 0.3; 0.4; 0.5
    l2penalty = 0 #0; 0.05; 0.1; 0.15; 0.2; 0.25; 0.3; 0.35; 0.4; 0.45; 0.5
    
    agent = DDPGagent(env, droprob, l2penalty)
    #ounoise = OUNoise(env.action_space)
    rdnoise = Randnormalnoise(env.action_space)
    
    rewards = []
    avg_rewards = []
    msg = []
    tot_info = []

    n_games = 1
    n_steps = 50 # n_steps*0.5s
    
    initial_beta=0.5
    final_beta=1
    delta_beta = final_beta - initial_beta
    k = delta_beta / n_games

    best_score = float('-inf')

#%%
    for episode in tqdm(range(n_games)):
        state = env.reset()
        #ounoise.reset()
        episode_reward = 0
        
        beta = max(initial_beta + k * (episode+1), 1)
        
        for step in range(n_steps):
            action = agent.get_action(state)
            #action = ounoise.get_action(action, step)
            action = rdnoise.get_action(action)
            new_state, reward, done, info = env.startNextStep(action)
            
            
            if episode == n_games - 1:
                tot_info.append(info)
   
            #calculate TD-error and save data
            agent.save(state, action, reward, new_state, done)
            
            if len(agent.memory) > batch_size:
                agent.update(batch_size, beta)

            state = new_state
            episode_reward += reward

            
            if done and env.zeroVelCounter >= 50:
                print("ending episode due to zero velocity")
                break

            # elif done:
            #     print('Collision might happened, end game')
            #     break
            
        
        rewards.append(episode_reward)
        avg_score = np.mean(rewards[-50:])
        avg_rewards.append(avg_score)

        if avg_score > best_score:
            best_score = avg_score
            agent.save_models()
            print('model saved')
        
        msg.append('episode {}, score {}, average score {}'.format(episode, round(episode_reward,4), round(avg_score,4)))
        print('episode {}, score {}, average score {}'.format(episode, round(episode_reward,4), round(avg_score,4)))

    
    # plt.plot(rewards)
    # plt.plot(avg_rewards)
    # plt.plot()
    # plt.xlabel('Episode')
    # plt.ylabel('Reward')
    # plt.show()
    env.stopSimulation("stop")
    
#%%
    # memory = np.array(agent.memory.buffer, dtype=object)
    # state_memory = memory[:,0]
    # action_memory = memory[:,1]
    # reward_memory = memory[:,2]
    # new_state_memory = memory[:,3]
    tot_info = np.array(tot_info, dtype = object)

    # np.save('state_memory.npy', state_memory)
    # np.save('action_memory.npy', action_memory)
    # np.save('new_state_memory.npy', new_state_memory)
    # np.save('reward_memory.npy', reward_memory)
    # np.save('msg_{}_PER_dp{}_wd{}.npy'.format(batch_size, str(droprob).replace('.', 'x'), str(l2penalty).replace('.', 'x')), np.array(msg))
    # np.save('tot_info.npy', tot_info)
