# %% initilization
import os
import sys

if 'SUMO_HOME' in os.environ:
    tools = os.path.join(os.environ['SUMO_HOME'], 'tools')
    sys.path.append(tools)
else:
    sys.exit("please declare environment variable 'SUMO_HOME'")
import traci
import traci.constants as tc
import matlab.engine as eng
import xml.etree.ElementTree as ET
import pandas as pd
import copy
import numpy as np
import time
import math
from pylab import sin, cos, exp
from ast import literal_eval
from random import choice
# %% ns3 import
import ns.applications
import ns.core
import ns.csma
import ns.internet
import ns.mobility
import ns.network
import ns.olsr
import ns.wifi
import ns.wave
import ns.lte
import ns.point_to_point
import ns.buildings

# %%inits
vehIDs = []
vehSpeed = {}
desiredVel = {}
cur_nodes = {}
nodesInfo = {}
predictionTime = 100
radarpenpct = 100
initial = 1
mainDirec = r"/home/cy/Documents/CoMoVe"
fileNm = "complxNode"
sumoBinary = "/home/cy/Downloads/sumo-1.1.0/bin/sumo-gui"
sumoCmd = [sumoBinary, "-c",
           r"/home/cy/Documents/CoMoVe/sumoFiles/" + fileNm + ".sumocfg",
           "--start",
           "--quit-on-end"]

if eng.find_matlab():
    eng = eng.connect_matlab(eng.find_matlab()[0])
else:
    sys.exit("no Matlab engine shared")

eng.eval("cd /home/cy/Documents/CoMoVe/mathWorksFiles/matlabFiles", nargout=0)
eng.eval("addpath('/home/cy/Documents/CoMoVe/mathWorksFiles/simulinkFiles')", nargout=0)
eng.eval("open_system('" + os.path.join(mainDirec, r"mathWorksFiles/simulinkFiles/radarDetectionModel") + "')",
         nargout=0)
simuName = "radarDetectionModel"
simCmd = "SimulationCommand"
cmds = ['start', 'continue', 'stop']
simucmd = "set_param('{}','{}','{}')"
gainIndex = [-1]
gainRange = [[-1.8, 2.5, -2.5, 2.4], [-2.0, 2.5, -2.5, 2.5]]


class Unbuffered(object):
    def __init__(self, stream):
        self.stream = stream

    def write(self, data):
        self.stream.write(data)
        self.stream.flush()

    def writelines(self, datas):
        self.stream.writelines(datas)
        self.stream.flush()

    def __getattr__(self, attr):
        return getattr(self.stream, attr)


# %% functions
def gainValues(idx):
    gains = gainRange[idx]
    kdminValue = gains[0]
    kdmaxValue = gains[1]
    kd_dx = (abs(kdminValue) + abs(kdmaxValue)) / predictionTime
    X = np.arange(kdminValue, kdmaxValue, kd_dx)
    Y = exp(-X ** 2)

    # Normalize the data to a proper PDF
    Y /= (kd_dx * Y).sum()

    # Compute the CDF
    CY = np.cumsum(Y * kd_dx)
    kd_values = 0.07 * (1 - CY)
    kvminValue = gains[2]
    kvmaxValue = gains[3]
    kv_dx = (abs(kvminValue) + abs(kvmaxValue)) / predictionTime
    X = np.arange(kvminValue, kvmaxValue, kv_dx)
    Y = exp(-X ** 2)

    # Normalize the data to a proper PDF
    Y /= (kv_dx * Y).sum()

    # Compute the CDF
    CY = np.cumsum(Y * kv_dx)
    kv1_values = 0.45 * (1 - CY)
    return (kd_values, kv1_values)


# %% adaptive cruise control
def ACC(vehID, carVelocity, targetDetected, targetDistance, targetRelVelocity, PredColMode=0, predTimems=0):
    print("ACC:{}".format(vehID))
    desiredSpeed = desiredVel[vehID]
    print(carVelocity, targetDetected, targetDistance, targetRelVelocity, PredColMode, predTimems)

    kv1 = 0.45
    kd = 0.07
    kdd = 0.01
    kv2 = 0.3
    kv1_b = kv1
    kd_b = kd
    # if PredColMode:
    #     sel_idx = [0 if int(targetDistance / abs(targetRelVelocity)) > 12 else 1][0]
    #     gainIndex[0] = sel_idx if gainIndex[0] == -1 else gainIndex[0]
    #     kd_values, kv1_values = gainValues(gainIndex[0])
    #     kv1_b = kv1_values[predTimems]
    #     kd_b = kd_values[predTimems]
    #
    # else:
    #     kv1_b = kv1
    #     kd_b = kd

    minAcc = 0
    maxAcc = 1
    minDec = 0
    maxDec = -2.45
    targetDist = targetDistance
    desiredDist = 0.6 * carVelocity
    desiredDist = desiredDist if desiredDist > 2 else 2

    distVh = (targetRelVelocity * kv1) + ((targetDist - desiredDist) * kd)
    distAcc1 = (targetDist - desiredDist) * kdd
    # saturating acceleration
    distAcc = distVh if minAcc <= distVh <= maxAcc else minAcc if distVh < minAcc else maxAcc
    distAcc1 = distAcc1 if minAcc <= distAcc1 <= maxAcc else minAcc if distAcc1 < minAcc else maxAcc

    speedAcc = (desiredSpeed - carVelocity) * kv2
    speedAcc = speedAcc if minAcc <= speedAcc <= maxAcc else minAcc if speedAcc < minAcc else maxAcc

    acc = [min(speedAcc, distAcc) if min(speedAcc, distAcc) > 0 else min(distAcc, distAcc1)][0]

    reqAcceleration = [acc if speedAcc > 0 else speedAcc][0]

    # distVh_bb = (targetRelVelocity*kv1_b)+((targetDist-desiredDist)*kd_b)

    distVh_b = (targetRelVelocity * kv1_b) + ((targetDist - desiredDist) * kd_b)

    # print("Velocity Weight:{},Distance Weight:{}".format((targetRelVelocity*kv1_b),((targetDist-desiredDist)*kd_b)))

    distDec = distVh_b if maxDec <= distVh_b <= minDec else maxDec if distVh_b < maxDec else minDec
    # distDecb = distVh_bb if maxDec<=distVh_bb<=minDec else maxDec if distVh_bb<maxDec else minDec
    reqDeceleration = [distDec if (-targetRelVelocity > 0) else 0][0]
    acceleration = [reqAcceleration if reqDeceleration >= 0 else reqDeceleration][0]

    if targetDetected and targetRelVelocity > 0.0:
        # calculating AEB
        ttc = targetDist / (-targetRelVelocity)

        if ttc <= 2.4 and (-targetRelVelocity) > 0:
            acceleration = -2.6

        elif ttc <= 1 and (-targetRelVelocity) > 0:
            acceleration = -9.5

    # print(reqAcceleration)
    # print(speedAcc)
    if targetDetected:
        # print("graph")
        # print(distVh_bb)
        return round(acceleration, 5)
    else:
        return round(speedAcc, 5)


# %%calcModifiedVelocity
# find pairs
def acctovel(acc, vel):
    newVel = vel + (0.1 * acc)
    return (newVel)


def validlanecheck(curveh, radveh):
    # if (
    #         ((curveh[1] != radveh[1]) and (radveh[1].split("_")[0] not in relatededges[curveh[1].split("_")[0]]))
    #         and
    #         (curveh[0] == radveh[0] or
    #          abs(curveh[0] - radveh[0]) == 180)
    # ):
    #     return 0
    # else:
    #     return 1
    try:
        if curveh[1] == radveh[1]:
            print("same lane")
            return (1)
        elif curveh[1] != radveh[1] and radveh[1].split("_")[0] in relatededges[curveh[1].split("_")[0]]:
            if abs(curveh[0] - radveh[0]) == 180:
                print("related lane opposite direction")
                return (0)
            else:
                print("related lane same direction")
                return (1)
        elif curveh[1] != radveh[1]:
            if (curveh[0] == radveh[0] or
                    abs(curveh[0] - radveh[0]) == 180):
                print("different lane opposite/same direction")
                return (0)
            else:
                print("different lane perpn. direction")
                return (1)
    except KeyError as e:
        print("KeyError:{}".format(e))
        return (1)


def calcsafeDistance(carVelocity):
    carVelocity = carVelocity * 3.6
    # print(carVelocity)
    normalizedVelocity = (carVelocity ** 2) / 100

    if 0 <= normalizedVelocity < 25:
        return (max(normalizedVelocity, 2))
    elif 25 <= normalizedVelocity < 81:
        return (max(normalizedVelocity - (carVelocity - 50), 2))
    elif normalizedVelocity >= 81:
        return (max(normalizedVelocity - 40), 2)


def getcaccparameter(curveh, neighvehs):
    print("CACC starts for:{}".format(curveh["vehID"]))
    carDistances = [[] * 1 for _ in range(len(neighvehs))]
    curDist = [0 for _ in range(len(neighvehs))]
    for idx, neighveh in enumerate(neighvehs):
        print("Lane Check for:{}".format(neighveh["vehID"]))
        if validlanecheck([curveh[k] for k in ["angle", "laneid"]],
                          [neighveh[k] for k in ["angle", "laneid"]]):
            neighvehdirec = math.degrees(
                math.atan2(neighveh["pos"][1] - curveh["pos"][1], neighveh["pos"][0] - curveh["pos"][0]))
            neighvehdirec = neighvehdirec if int(neighvehdirec) != 0 else 360.0
            if abs(curveh["angle"] - neighvehdirec) == 90 or (90 < neighvehdirec < 270):
                print("{} is behind the curent vehicle:{}".format(neighveh["vehID"], curveh["vehID"]))
            else:
                curDist[idx] = np.linalg.norm(curveh['pos'] - neighveh['pos'])
                for i in range(int(predictionTime)):
                    carDistances[idx].append(
                        np.linalg.norm((curveh['predposition'][i] - neighveh['predposition'][i])))

    safeDist = calcsafeDistance(curveh['speed'])
    minCardist = []
    for i in carDistances:
        if i:
            minCardist.append(min(i))
        else:
            minCardist.append(10000000)
    minCardist = [[] if min(minCardist) == 10000000 else minCardist][0]
    if minCardist:
        nearbyCarIdx = np.argmin(minCardist)
        params = {"mode": "CACC", "accel": 0, "cur_speed": curveh['speed'], "distance": 0,
                  "vehID": curveh["vehID"], "neighVeh": neighvehs[nearbyCarIdx]["vehID"],
                  "predicted": 0, "detection": 0}
        print("CACC Neighbouring Vehicle:{}".format(params["neighVeh"]))
        if curDist[nearbyCarIdx] <= safeDist:
            print("Collision Detected")
            params["accel"], params["distance"] = ACC(curveh["vehID"], curveh["speed"], 1, curDist[nearbyCarIdx],
                                                      neighvehs[nearbyCarIdx]["speed"] - curveh["speed"]), \
                                                  curDist[nearbyCarIdx]
            params["detection"] = 1

        elif min(minCardist) < safeDist < curDist[nearbyCarIdx]:
            print("Predicted Collision")
            predTimems = (min([i for i, x in enumerate(carDistances[nearbyCarIdx]) if x < safeDist]))
            params["accel"], params["distance"] = ACC(curveh["vehID"], curveh["speed"], 1, min(minCardist),
                                                      neighvehs[nearbyCarIdx]["speed"] - curveh["speed"], 1,
                                                      predTimems), \
                                                  curDist[nearbyCarIdx]
            params["detection"] = 1
            params["predicted"] = 1

        else:
            print("No Collision")
            params["accel"], params["distance"] = ACC(curveh["vehID"], curveh["speed"], 0, curDist[nearbyCarIdx],
                                                      neighvehs[nearbyCarIdx]["speed"] - curveh["speed"]), \
                                                  curDist[nearbyCarIdx]

        return params
    else:
        return {"mode": "CACC", "accel": 2, "cur_speed": 0, "distance": 0,
                "vehID": "", "neighVeh": ""}


def findvehpairs(vehpairs):
    output = []
    excludeveh = []
    for veh in vehpairs.keys():
        if veh not in excludeveh:
            if vehpairs[veh]["detection"]:
                try:
                    if vehpairs[vehpairs[veh]["neighVeh"]]["neighVeh"] == veh:
                        excludeveh.append(vehpairs[veh]["neighVeh"])
                        # currently vehicle selection is based on acceleration to collision and
                        # given importance to non predicted collisions
                        #
                        # or vehpairs[vehpairs[veh]["neighVeh"]]["predicted"]) and not vehpairs[veh]["predicted"]
                        if vehpairs[veh]["accel"] < vehpairs[vehpairs[veh]["neighVeh"]]["accel"]:
                            newvel = acctovel(vehpairs[veh]["accel"], vehpairs[veh]["cur_speed"])
                            output.append({"vehID": veh, "newVelocity": newvel, "acceleration": vehpairs[veh]["accel"]})
                        else:
                            newvel = acctovel(vehpairs[vehpairs[veh]["neighVeh"]]["accel"],
                                              vehpairs[vehpairs[veh]["neighVeh"]]["cur_speed"])
                            output.append({"vehID": vehpairs[veh]["neighVeh"],
                                           "newVelocity": newvel,
                                           "acceleration": vehpairs[vehpairs[veh]["neighVeh"]]["accel"]})
                    else:
                        newvel = acctovel(vehpairs[veh]["accel"], vehpairs[veh]["cur_speed"])
                        output.append({"vehID": veh, "newVelocity": newvel, "acceleration": vehpairs[veh]["accel"]})
                except KeyError:
                    print("Neighbouring vehicle not available in actionable output")
            else:
                newvel = acctovel(vehpairs[veh]["accel"], vehpairs[veh]["cur_speed"])
                output.append({"vehID": veh, "newVelocity": newvel, "acceleration": vehpairs[veh]["accel"]})

    return (output)


def collisioncheck(vehinfo):
    print('------------')
    print("Collision Detection Started")
    actionableOuput = {}
    vehdata = {i[-1]: {"vehID": i[-1], "speed": i[0], "angle": i[1], "pos": np.array(i[2]),
                       "radar": i[3][0] if i[3][0][-1] != -1.0 else i[3][-1], "laneid": i[4]} for i in vehinfo}
    for i in vehdata.keys():
        directionVector = np.array(
            [round(sin(math.radians(vehdata[i]["angle"])), 2), round(cos(math.radians(vehdata[i]["angle"])), 2), 0]) * \
                          (vehdata[i]["speed"] / 10)
        vehdata[i]['predposition'] = [vehdata[i]["pos"] + (np.array(directionVector) * k) for k in
                                      range(1, int(predictionTime) + 1)]

    for i in vehdata.keys():
        print('------------')
        excludeveh = [i]
        print("RADAR")
        radarparam = {"mode": "RADAR", "detection": 0, "cur_speed": vehdata[i]["speed"], "accel": 2,
                      "distance": 0, "vehID": i, "neighVeh": "", "predicted": 0}
        if vehdata[i]["radar"][-1] != -1 and i not in vehworadar:
            if vehdata[i]["radar"][-1]:
                radarparam["detection"] = 1
                radarparam["distance"] = vehdata[i]["radar"][0]
                radarparam["neighVeh"] = "veh" + str(int(vehdata[i]["radar"][-1]) - 1)
                excludeveh.append(radarparam["neighVeh"])
                detectionparam = validlanecheck([vehdata[i][k] for k in ["angle", "laneid"]],
                                                [vehdata[radarparam["neighVeh"]][k] for k in ["angle", "laneid"]])
                radarparam["accel"] = ACC(i, vehdata[i]["speed"], detectionparam,
                                          vehdata[i]["radar"][0], vehdata[i]["radar"][1])
            else:
                radarparam["accel"] = ACC(i, vehdata[i]["speed"], 0,
                                          vehdata[i]["radar"][0], vehdata[i]["radar"][1])
        caccveh = [set(vehdata.keys()) - set(excludeveh)][0]
        caccparams = getcaccparameter(vehdata[i], [vehdata[k] for k in caccveh])
        actionableOuput[i] = [radarparam if radarparam["accel"] <= caccparams["accel"] else caccparams][0]
        print('------------')
    cc_output = findvehpairs(actionableOuput)
    print("Collision Check Output:{}".format(cc_output))
    return cc_output


def infracallback(sink):
    try:
        addr = ns.network.Address()
        data = sink.RecvFrom(addr).GetString().split(":")
        if len(nodesInfo[int(data[0])]["data"]) == 1:
            nodesInfo[int(data[0])]["data"].pop(0)
        nodesInfo[int(data[0])]["data"].append([round(float(data[1]), 3), round(float(data[2]), 3)])
        nodesInfo[int(data[0])]["data"][0].extend([literal_eval(data[3]), literal_eval(data[4]), data[5]])
        nodesInfo[int(data[0])]["data"][0].append(nodesInfo[int(data[0])]["vehPair"])
        nodesInfo[int(data[0])]["address"] = addr
        availCars = [sum(1 for i in nodesInfo.keys() if len(nodesInfo[i]["data"]))][0]
        if availCars == len(vehIDs):
            print([nodesInfo[i]["data"][0] for i in nodesInfo.keys()])
            updateinfo = collisioncheck([nodesInfo[i]["data"][0] for i in nodesInfo.keys()])
            [nodesInfo[i]["data"].pop() for i in nodesInfo.keys()]
            for vehinfo in updateinfo:
                datatoupdate = vehinfo["vehID"] + ":" + str(vehinfo["newVelocity"])
                print(datatoupdate)
                srcnodeid = [i for i in nodesInfo.keys() if nodesInfo[i]["vehPair"] == vehinfo["vehID"]][0]
                sink.SendTo(ns.network.Packet(datatoupdate, len(datatoupdate) + 1), len(datatoupdate) + 1,
                            nodesInfo[srcnodeid]["address"])
    except Exception as e:
        print("error in recieving:{}".format(e))


def nodescallback(sink):
    updatevehid, updatedvelocity = sink.Recv().GetString().split(":")
    print("update Received for:{}".format(updatevehid))
    traci.vehicle.setSpeed(updatevehid, float(updatedvelocity))


def generatetraffic(src, vehInfo, radarObjects, idx):
    try:
        pos = traci.simulation.convert3D(
            vehInfo[tc.VAR_ROAD_ID],
            vehInfo[tc.VAR_LANEPOSITION],
            vehInfo[tc.VAR_LANE_INDEX], 'False')
        pos = [round(i, 4) for i in pos]
        data = str(src.GetNode().GetId()) + ":" + \
               str(vehInfo[tc.VAR_SPEED]) + ":" + \
               str(vehInfo[tc.VAR_ANGLE]) + ":" + \
               str(pos) + ":" + \
               str(radarObjects) + ":" + \
               str(vehInfo[tc.VAR_LANE_ID]) + ":" + str(idx)

        # print(data)
        obj = ueNodes.Get(idx).GetObject(ns.mobility.MobilityModel.GetTypeId())
        obj.SetPosition(ns.core.Vector(int(pos[0]), int(pos[1]), int(pos[2])))
        b_helper.MakeMobilityModelConsistent()
        src.Send(ns.network.Packet(data, len(data) + 1))
    except Exception as e:
        print("error in sending:{}".format(e))


# %% actors waypoint update based on SUMO values
def nextsimulationstep():
    vehStatus = {}
    try:
        print("next sim")
        global vehIDs, initial
        if traci.simulation.getMinExpectedNumber() > 0:
            simStatus = 1
            actorCurStatus = "%Actors_Start\n"
            if traci.simulation.getLoadedNumber() > 0:
                vehIDs.extend(traci.simulation.getLoadedIDList())
                for i in vehIDs:
                    traci.vehicle.subscribe(i, (
                        tc.VAR_SPEED, tc.VAR_ANGLE, tc.VAR_ROAD_ID, tc.VAR_LANEPOSITION, tc.VAR_LANE_INDEX,
                        tc.VAR_LANE_ID))
            # if initial:
            #     initial = 0
            #     traci.simulationStep()
            # else:
            traci.simulationStep()
            listOfVeh = list(traci.vehicle.getIDList())
            if vehIDs != listOfVeh:
                del vehIDs[:]
                vehIDs.extend(listOfVeh)
            arrivedvehs = list(traci.simulation.getArrivedIDList())
            [nodesInfo.pop(k, None) for k in nodesInfo.keys() if nodesInfo[k]["vehPair"] in arrivedvehs]
            for curVehID in vehIDs:
                vehStatus[curVehID] = traci.vehicle.getSubscriptionResults(curVehID)
                [posy, posx, posz] = traci.simulation.convert3D(
                    vehStatus[curVehID][tc.VAR_ROAD_ID],
                    vehStatus[curVehID][tc.VAR_LANEPOSITION],
                    vehStatus[curVehID][tc.VAR_LANE_INDEX], 'False')
                [posy, posx, posz] = [round(posy, 2), round(posx, 2), round(posz, 2)]
                curVehSpeed = vehStatus[curVehID][tc.VAR_SPEED]
                vehSpeed[curVehID] = curVehSpeed
                curVehAngle = vehStatus[curVehID][tc.VAR_ANGLE]
                directionVector = np.array(
                    [round(sin(math.radians(curVehAngle)), 2), round(cos(math.radians(curVehAngle)), 2), 0]) * \
                                  curVehSpeed

                [desty, destx, destz] = np.array([posy, posx, posz]) + directionVector

                actorCurStatus = actorCurStatus + curVehID \
                                 + "= vehicle(scenario,'ClassID'," \
                                 + str(actorConstants["Car"]) + ",'Position',[" \
                                 + str(posx) + " " + str(posy) + " " + str(posz) \
                                 + "],'Yaw', " + str(curVehAngle) + ");\n"
                if int(curVehSpeed):
                    actorCurStatus = actorCurStatus + "waypoints = [" \
                                     + str(posx) + " " + str(posy) + " " + str(posz) + ";" \
                                     + str(destx) + " " + str(desty) + " " + str(destz) + "];\n"
                    actorCurStatus = actorCurStatus + "speed = " + str(curVehSpeed) + ";\n"
                    actorCurStatus = actorCurStatus + "trajectory(" + curVehID + ",waypoints,speed);\n\n"

            actorCurStatus = actorCurStatus + "%Actors_End"
            with open(os.path.join(mainDirec, r"mathWorksFiles/matlabFiles/" + fileNm + "DrivingScenario.m"), 'r') as f:
                matlabFileContent = f.readlines()
            for idx, x in enumerate(matlabFileContent):
                if 'Actors_Start' in x:
                    startIdx = idx
                elif 'Actors_End' in x:
                    endIdx = idx
                    break

            newStatus = ""
            newStatus = newStatus.join(matlabFileContent[:startIdx])
            newStatus = newStatus + actorCurStatus
            newStatus = newStatus + newStatus.join(matlabFileContent[endIdx:-1])
            return (newStatus, simStatus, vehStatus)
        else:
            simStatus = 0
            traci.close()
            return ("No Vehicle Info. Simulation Ended", simStatus, {})
    except Exception as e:
        print("Something went wrong: {}".format(e))
        return ("error", 0, {})


def runsimulation():
    global vehIDs
    try:
        newVehStatus, sim_status, vehicleinfo = nextsimulationstep()
        vehicleRadar = {i: [] for i in vehIDs}
        if sim_status:
            with open(os.path.join(mainDirec, r"mathWorksFiles/matlabFiles/" + fileNm + "DrivingScenario.m"), 'w') as f:
                f.write(newVehStatus)
                f.close()
            cmd = "scenario = " + fileNm + "DrivingScenario();"
            eng.eval(cmd, nargout=0)
            print(vehIDs)
            for idx, val in enumerate(vehIDs):
                if val not in vehworadar:
                    eng.workspace["egoCarID"] = idx + 1
                    # eng.eval("set_param('radarDetectionModel/Scenario Reader','EgoVehicleActorID','"+str(idx+1)+"')",
                    # nargout=0)
                    eng.eval(simucmd.format(simuName, simCmd, cmds[0]), nargout=0)
                    time.sleep(0.5)
                    eng.eval("radarValues = out.measurement([1,4,7],:,:);", nargout=0)
                    vehVal = eng.workspace['radarValues']
                    vehicleRadar[val] = [[i[0][0] for i in vehVal],
                                         [i[0][1] for i in vehVal]]
                else:
                    vehicleRadar[val] = [[0, 0, 0], [0, 0, 0]]
                ns.core.Simulator.ScheduleNow(generatetraffic, cur_nodes[idx], vehicleinfo[val], vehicleRadar[val], idx)
            ns.core.Simulator.Schedule(ns.core.Seconds(0.1), runsimulation)

        else:
            print(newVehStatus)
            ns.core.Simulator.Stop()
    except Exception as ex:
        print("Run Simulation error:{}".format(ex))
        traci.close()
        ns.core.Simulator.Stop()


def findedges(val1, val2, ref, mainaxis, refaxis):
    matchednodes = list(nodes_df[((nodes_df[mainaxis] >= str(val1)) | (nodes_df[mainaxis] <= str(val1))) & (
            nodes_df[refaxis] == str(ref))]["id"])
    matchednodes.extend(list(nodes_df[((nodes_df[mainaxis] >= str(val2)) | (nodes_df[mainaxis] <= str(val2))) & (
            nodes_df[refaxis] == str(ref))]["id"]))
    uniqnodes = set(matchednodes)
    matchededges = []
    for i in list(uniqnodes):
        matchededges.extend(
            edges_df[(edges_df['to'].apply(lambda nd: 1 if nd in uniqnodes else 0) == 1)
                     & (edges_df["frm"] == i)]["id"].values)
    return (list(set(matchededges)))


# %% scenario creation
tree = ET.ElementTree(file=os.path.join(mainDirec, "sumoFiles/" + fileNm + ".nod.xml"))
nodes = list(tree.getroot())
nodeValues = [i.attrib for i in nodes if 'node' in i.tag]
nodes_df = pd.DataFrame(nodeValues)
tree = ET.ElementTree(file=os.path.join(mainDirec, r"sumoFiles/" + fileNm + ".edg.xml"))
edges = list(tree.getroot())
edgeValues = [i.attrib for i in edges if 'edge' in i.tag]
edges_df = pd.DataFrame(edgeValues)
edges_df.columns = edges_df.columns.str.replace('from', 'frm')
dupEdges = []
roadDict = {'roadEdges': [], 'lane': 0, 'speed': 0}
roads = []
for edge in edges_df.itertuples():
    if edge.Index not in dupEdges:
        curEdge = copy.deepcopy(roadDict)
        startNode = nodes_df[nodes_df['id'] == edge.frm]
        endNode = nodes_df[nodes_df['id'] == edge.to]
        curEdge["roadEdges"].append(
            (startNode.iloc[0]["y"]) + " "
            + startNode.iloc[0]["x"] + " "
            + "0;" + (endNode.iloc[0]["y"]) + " "
            + endNode.iloc[0]["x"] + " "
            + "0")
        curEdge["lane"] = 1
        curEdge["speed"] = edge.speed
        idx = list(edges_df[(edges_df['frm'] == edge.to)
                            & (edges_df['to'] == edge.frm)].index)
        if idx:
            curEdge["lane"] += 1
            dupEdges.extend(idx)
        roads.append(curEdge)

txt = "function scenario = " + fileNm + "DrivingScenario()\n"
txt = txt + "scenario = drivingScenario('SampleTime', 0.1);\n"

for i in roads:
    txt = txt + "\nroadCenters = [" + i["roadEdges"][0] + "];\n"
    txt = txt + "lm = laneMarking('Dashed','Length',1.5,'Space',3);\n"
    txt = txt + \
          "laneSpecification = lanespec(" + str(i["lane"]) + \
          ", 'Width', 2.925,'Marking',lm);\n"
    txt = txt + "road(scenario, roadCenters, 'Lanes', laneSpecification);\n"

with open(os.path.join(mainDirec, r"mathWorksFiles/matlabFiles/" + fileNm + "DrivingScenario.m"), 'w') as f:
    f.write(txt)
    f.close()

# %% actors creation
actorConstants = {"Car": 1, "Truck": 2, "Bicycle": 3, "Pedestrian": 4, "Barrier": 5}

tree = ET.ElementTree(file=os.path.join(mainDirec, "sumoFiles/" + fileNm + ".rou.xml"))
nodes = list(tree.getroot())
routeValues = [i.attrib for i in nodes if 'route' in i.tag]
vTypes = [i.attrib for i in nodes if 'vType' in i.tag]
vehicles = [i.attrib for i in nodes if 'vehicle' in i.tag]
vehicleIDs = [i["id"] for i in vehicles]
actorTxt = "%Actors_Start\n"

for route in routeValues:
    [vehID, speed] = [[i['id'], i["departSpeed"]] for i in vehicles if i["route"] == route["id"]][0]
    desiredVel[vehID] = float(speed)
    [frm, to] = [[i["from"], i["to"]] for i in edgeValues if i["id"] == route["edges"].split(" ")[0]][0]
    [orgy, orgx] = [[i["x"], i["y"]] for i in nodeValues if i["id"] == frm][0]
    [desty, destx] = [[i["x"], i["y"]] for i in nodeValues if i["id"] == to][0]
    actorTxt = actorTxt + vehID \
               + "= vehicle(scenario,'ClassID'," \
               + str(actorConstants["Car"]) + ",'Position',[" \
               + str(orgx) + " " + str(orgy) + " 0]);\n"
    if int(speed):
        actorTxt = actorTxt + "waypoints = [" \
                   + str(orgx) + " " + str(orgy) + " 0;" \
                   + str(destx) + " " + str(desty) + " 0];\n"
        actorTxt = actorTxt + "speed = " + speed + ";\n"
        actorTxt = actorTxt + "trajectory(" + vehID + ",waypoints,speed);\n"

actorTxt = actorTxt + "%Actors_End\n\n"
with open(os.path.join(mainDirec, r"mathWorksFiles/matlabFiles/" + fileNm + "DrivingScenario.m"), 'a') as f:
    f.write(actorTxt)
    f.close()

vehworadar = ["veh1"]
# veh = len(vehIDs)
# noradarvehcnt = int(veh-math.floor(veh*(radarpenpct/100.0)))
# while len(vehworadar)< noradarvehcnt:
#     x = choice(range(0,veh))
#     if "veh"+str(x) not in vehworadar:
#         vehworadar.append("veh"+str(x))
relatededges = {}
for i in edges_df.iterrows():
    x1, y1 = list(nodes_df[nodes_df["id"] == i[1].frm][["x", "y"]].values)[0]
    x2, y2 = list(nodes_df[nodes_df["id"] == i[1].to][["x", "y"]].values)[0]
    if x1 == x2:
        relatededges[i[1].id] = findedges(y1, y2, x1, "y", "x")
    elif y1 == y2:
        relatededges[i[1].id] = findedges(x1, x2, y1, "x", "y")
    else:
        relatededges[i[1].id] = []

sys.stdout = Unbuffered(sys.stdout)
# %% ns3 codes
cmd = ns.core.CommandLine()
cmd.RxGain = 1
numnodes = len(set(vehicleIDs)) + 1

ns.core.GlobalValue.Bind("SimulatorImplementationType", ns.core.StringValue("ns3::RealtimeSimulatorImpl"))

lteHelper = ns.lte.LteHelper()
epcHelper = ns.lte.PointToPointEpcHelper()
lteHelper.SetEpcHelper(epcHelper)
pgw = epcHelper.GetPgwNode()

remoteHostContainer = ns.network.NodeContainer()
remoteHostContainer.Create(1)
remoteHost = remoteHostContainer.Get(0)
internet = ns.internet.InternetStackHelper()
internet.Install(remoteHostContainer)

p2ph = ns.point_to_point.PointToPointHelper()
p2ph.SetDeviceAttribute("DataRate", ns.core.StringValue("100Gbps"))
p2ph.SetDeviceAttribute("Mtu", ns.core.StringValue("1500"))
p2ph.SetChannelAttribute("Delay", ns.core.StringValue("2ms"))
internetDevices = p2ph.Install(pgw, remoteHost)
ipAddrs = ns.internet.Ipv4AddressHelper()
ipAddrs.SetBase(ns.network.Ipv4Address("1.0.0.0"), ns.network.Ipv4Mask("255.0.0.0"))
internetIpIfaces = ipAddrs.Assign(internetDevices)
remoteHostAddr = internetIpIfaces.GetAddress(1)

ipv4RoutingHelper = ns.internet.Ipv4StaticRoutingHelper()

remoteHostStaticRouting = ipv4RoutingHelper.GetStaticRouting(remoteHost.GetObject(ns.internet.Ipv4.GetTypeId()))
remoteHostStaticRouting.AddNetworkRouteTo(ns.network.Ipv4Address("7.0.0.0"), ns.network.Ipv4Mask("255.0.0.0"), 1)

# building
# x_min = -10.0
# x_max = 0.0
# y_min = -20.0
# y_max = 0.0
# z_min = 0.0
# z_max = 10.0
# b = ns.buildings.Building()
# b.SetBoundaries(ns.mobility.Box(x_min, x_max, y_min, y_max, z_min, z_max))
# b.SetBuildingType(b.Residential)
# b.SetExtWallsType(b.ConcreteWithWindows)
# b.SetNFloors(3)
# b.SetNRoomsX(3)
# b.SetNRoomsY(2)

numeNbnodes = 1
numuenodes = numnodes - 1
eNbNodes = ns.network.NodeContainer()
eNbNodes.Create(numeNbnodes)
ueNodes = ns.network.NodeContainer()
ueNodes.Create(numuenodes)

# print(dir(ns.lte))
# nodesPos = [ns.core.Vector(-500,0,0),ns.core.Vector(0,-300,0),ns.core.Vector(-5,0,0)]
mobility1 = ns.mobility.MobilityHelper()
b_helper = ns.buildings.BuildingsHelper()
mobility1.SetMobilityModel("ns3::ConstantPositionMobilityModel")
mobility1.Install(eNbNodes)
b_helper.Install(eNbNodes)
obj = eNbNodes.Get(0).GetObject(ns.mobility.MobilityModel.GetTypeId())
obj.SetPosition(ns.core.Vector(0, 0, 0))

mobility = ns.mobility.MobilityHelper()
mobility.SetMobilityModel("ns3::ConstantPositionMobilityModel")
mobility.Install(ueNodes)
b_helper.Install(ueNodes)
b_helper.MakeMobilityModelConsistent()

propagationLossModel = ns.buildings.HybridBuildingsPropagationLossModel()
propagationLossModel.SetAttribute("ShadowSigmaOutdoor", ns.core.DoubleValue(0.0))
propagationLossModel.SetAttribute("ShadowSigmaIndoor", ns.core.DoubleValue(0.0))
propagationLossModel.SetAttribute("ShadowSigmaExtWalls", ns.core.DoubleValue(0.0))
propagationLossModel.SetEnvironment(0)

enbDevs = lteHelper.InstallEnbDevice(eNbNodes)
ueDevs = lteHelper.InstallUeDevice(ueNodes)

internet.Install(ueNodes)
ueIpIface = epcHelper.AssignUeIpv4Address(ueDevs)

for i in range(numuenodes):
    ueNode = ueNodes.Get(i)
    ueStaticRouting = ipv4RoutingHelper.GetStaticRouting(ueNode.GetObject(ns.internet.Ipv4.GetTypeId()))
    ueStaticRouting.SetDefaultRoute(epcHelper.GetUeDefaultGatewayAddress(), 1)

lteHelper.Attach(ueDevs, enbDevs.Get(0))

sink = ns.network.Socket.CreateSocket(remoteHost, ns.core.TypeId.LookupByName("ns3::UdpSocketFactory"))
local = ns.network.InetSocketAddress(ns.network.Ipv4Address.GetAny(), 9)
sink.Bind(local)
sink.SetRecvCallback(infracallback)

for nd in range(numuenodes):
    source = ns.network.Socket.CreateSocket(ueNodes.Get(nd), ns.core.TypeId.LookupByName("ns3::UdpSocketFactory"))
    nodesInfo[int(source.GetNode().GetId())] = {"data": [], "address": [], "vehPair": vehicleIDs[nd]}
    sinkaddress = ns.network.InetSocketAddress(remoteHostAddr, 9)
    address = ns.network.Ipv4Address.GetAny()
    local = ns.network.InetSocketAddress(address, 9)
    source.Bind(local)
    source.Connect(sinkaddress)
    source.SetRecvCallback(nodescallback)
    cur_nodes[nd] = source

print("starting SUMO")
traci.start(sumoCmd)
ns.core.Simulator.Schedule(ns.core.Seconds(0.1), runsimulation)
ns.core.Simulator.Run()
ns.core.Simulator.Destroy()
