# -*- coding: utf-8 -*-
"""
Created on Tue Jan 26 16:00:42 2021

@author: wky
"""

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.stats import norm
import matplotlib.mlab as mlab

def getepireward(msg, n_steps):
    epireward = []
    avgepireward = []
    for i in range(len(msg)):
        r = float(msg[i].replace(',', '').split(' ')[3])
        rewd = round(r, 4)
        avgrewd = round(r /  n_steps, 4)
        epireward.append(rewd)
        avgepireward.append(avgrewd)
    
    return epireward, avgepireward

def getleadspeed(leadacc, initialspeed, dt):
    leadspeed = np.zeros((len(leadacc)))
    
    for i in range(len(leadacc)):
        leadspeed[i] = sum(leadacc[0:i])*dt + initialspeed
    
    return leadspeed        

#%%
n_games = 1
n_steps = 50
dt = 0.1
#msg = np.load('msg.npy')
#info_memory = np.load('info_memory.npy')
tot_info = np.load('tot_info.npy', allow_pickle=True)

#info = np.reshape(info_memory,(-1,7))
tot_info = np.reshape(tot_info,(-1,8))
leadVehSpeeds = getleadspeed(tot_info[:,2], initialspeed = tot_info[0,6], dt = dt)

#%% reward per episode

x = list(range(n_games))
epi_rewd, avg_epi_rewd= getepireward(msg, n_steps)
plt.figure()
plt.plot(x, avg_epi_rewd)
plt.ylim([-0.5,1.5])
plt.xlim([-10, 210])
plt.grid()
plt.ylabel('Average Reward', fontsize = 15)
plt.xlabel('Simulations', fontsize = 15)
plt.title('Average reward per simulation', fontsize = 15)
plt.annotate(text='x=194, y=0.62',xy=(194, 0.623),xytext=(185, 1.2),weight='bold',color='black',
             arrowprops=dict(arrowstyle='->',connectionstyle='arc3',color='black'),
             bbox=dict(boxstyle='round,pad=0.5', fc='white', ec='k',lw=1 ,alpha=0.4))

#%%Results
xrange = [-1, n_steps*0.5+1]
x = np.arange(0, n_steps*0.5, 0.1)
plt.figure()
plt.suptitle('RL Agent Performance', fontsize = 13)

#Velocity
plt.subplot(321)
plt.plot(x, tot_info[0:n_steps*5,0], label = 'EgoVehicle')
plt.plot(x, leadVehSpeeds[0:n_steps*5], label = 'LeadVehicle')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Velocity (m/s)', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Velocity Trend', fontsize = 13)
plt.legend(loc=4)

#Headway
plt.subplot(322)
plt.plot(x, tot_info[0:n_steps*5,1], label = 'Headway')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Headway (sec)', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Headway Trend', fontsize = 13)

#Acceleration
plt.subplot(323)
plt.plot(x, tot_info[0:n_steps*5,3], label = 'EgoVehicle')
plt.plot(x, tot_info[0:n_steps*5,2], label = 'LeadVehicle')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Acceleration (m/s^2)', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Acceleration Trend', fontsize = 13)
plt.legend(loc=4)

#Spacing
spacing = np.multiply(tot_info[0:n_steps*5,0], tot_info[0:n_steps*5,1])
plt.subplot(324)
plt.plot(x, spacing, label = 'Spacing')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Spacing (m)', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Spacing Trend', fontsize = 13)

#Jerk
plt.subplot(325)
plt.plot(x, tot_info[0:n_steps*5,5], label = 'Jerk')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Jerk (m/s^3)', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Jerk Trend', fontsize = 13)

#Slip
plt.subplot(326)
plt.plot(x, tot_info[0:n_steps*5,4], label = 'Slip')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Slip', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Slip Trend', fontsize = 13)

#%%
#Distribution of reward without dynamic coefficients
split_rewdlist = tot_info[:, 7].tolist()
split_rewd = np.zeros((len(split_rewdlist), 4))
for i in range(len(split_rewdlist)):
    split_rewd[i,0] = split_rewdlist[i][1]/split_rewdlist[i][0][0]*sum(split_rewdlist[i][0]) #saferwd
    split_rewd[i,1] = split_rewdlist[i][2]/split_rewdlist[i][0][1]*sum(split_rewdlist[i][0]) #speedrwd
    split_rewd[i,2] = split_rewdlist[i][3]/split_rewdlist[i][0][2]*sum(split_rewdlist[i][0]) #stablerwd
    split_rewd[i,3] = split_rewdlist[i][4]/split_rewdlist[i][0][3]*sum(split_rewdlist[i][0]) #cmfortrwd

xrange = [-1, n_steps*0.5+1]
x = np.arange(0, n_steps*0.5, 0.1)
plt.figure()
plt.suptitle('Distribution of reward between the components without dynamic coefficients', fontsize = 13)

#Safe Reward
plt.subplot(411)
plt.plot(x, split_rewd[0:n_steps*5,0], label = 'Safe Reward')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Reward', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Safe Reward', fontsize = 13)

#Speed Reward
plt.subplot(412)
plt.plot(x, split_rewd[0:n_steps*5,1], label = 'Speed Reward')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Reward', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Speed Reward', fontsize = 13)

#Stablility Reward
plt.subplot(413)
plt.plot(x, split_rewd[0:n_steps*5,2], label = 'Stability Reward')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Reward', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Stability Reward', fontsize = 13)

#Comfort Reward
plt.subplot(414)
plt.plot(x, split_rewd[0:n_steps*5,3], label = 'Comfort Reward')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Reward', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Comfort Reward', fontsize = 13)

#%%
#Distribution of reward
split_rewdlist = tot_info[:, 7].tolist()
split_rewd = np.zeros((len(split_rewdlist), 4))
for i in range(len(split_rewdlist)):
    split_rewd[i,:] = split_rewdlist[i][1:5]

xrange = [-1, n_steps*0.5+1]
x = np.arange(0, n_steps*0.5, 0.1)
plt.figure()
plt.suptitle('Distribution of reward between the components', fontsize = 13)

#Total Reward
plt.subplot(511)
plt.plot(x, split_rewd[0:n_steps*5,0]+split_rewd[0:n_steps*5,1]+split_rewd[0:n_steps*5,2]+split_rewd[0:n_steps*5,3], label = 'Total Reward')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Reward', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Total Reward', fontsize = 13)

#Safe Reward
plt.subplot(512)
plt.plot(x, split_rewd[0:n_steps*5,0], label = 'Safe Reward')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Reward', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Safe Reward', fontsize = 13)

#Speed Reward
plt.subplot(513)
plt.plot(x, split_rewd[0:n_steps*5,1], label = 'Speed Reward')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Reward', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Speed Reward', fontsize = 13)

#Stablility Reward
plt.subplot(514)
plt.plot(x, split_rewd[0:n_steps*5,2], label = 'Stablility Reward')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Reward', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Stablility Reward', fontsize = 13)

#Comfort Reward
plt.subplot(515)
plt.plot(x, split_rewd[0:n_steps*5,3], label = 'Comfort Reward')
plt.xlim(xrange)
plt.grid()
plt.ylabel('Reward', fontsize = 12)
plt.xlabel('Time (sec)', fontsize = 12)
plt.title('Comfort Reward', fontsize = 13)

#%%Headway & Jerk Histogram

hw = tot_info[0:n_steps*5, 1].astype('float')
jerk = tot_info[0:n_steps*5, 5].astype('float')
n_bins = 25
plt.figure()
plt.suptitle('Headway & Jerk Histogram', fontsize = 13)

#Headway
plt.subplot(121)
n, bins, patches = plt.hist(hw, n_bins, rwidth=1, edgecolor='k', density = True)
plt.grid()
plt.ylabel('Frequency', fontsize = 12)
plt.xlabel('Headway (sec)', fontsize = 12)
plt.title('Histogram of Headway', fontsize = 13)
(mu, sigma) = norm.fit(hw)
plt.annotate(text='Normal Distribution Fit \n Mean = {} \n Std = {}'.format(round(mu,4), round(sigma,4)), 
             xy=(mu, sigma),xytext=(5, 0.6),weight='bold',color='black',
             bbox=dict(boxstyle='round,pad=0.5', fc='white', ec='k',lw=1 ,alpha=0.4))

#Jerk Histogram
plt.subplot(122)
n, bins, patches = plt.hist(jerk, n_bins, rwidth=1, edgecolor='k', density = True)
plt.grid()
plt.ylabel('Frequency', fontsize = 12)
plt.xlabel('Jerk (m/sec^3)', fontsize = 12)
plt.title('Histogram of Jerk', fontsize = 13)
(mu, sigma) = norm.fit(jerk)
plt.annotate(text='Normal Distribution Fit \n Mean = {} \n Std = {}'.format(round(mu,4), round(sigma,4)), 
             xy=(mu, sigma),xytext=(0.5, 0.5),weight='bold',color='black',
             bbox=dict(boxstyle='round,pad=0.5', fc='white', ec='k',lw=1 ,alpha=0.4))










