import torch as T
import torch.autograd
import torch.optim as optim
import sys
import numpy as np
import pandas as pd
import CoMoVe_RL_MultipleSteps_option2 as frmwrk
import time
from tqdm import tqdm
import random
import matplotlib.pyplot as plt
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.autograd
from torch.autograd import Variable
import os

seed = 0

#%%
class Memory:
    def __init__(self, max_size, batch_size, num_states, num_actions, savedBuffers={}):
        
        random.seed(seed)
        np.random.seed(seed)
        
        # Number of "experiences" to store at max
        self.buffer_capacity = max_size
        # Num of tuples to train on.
        self.batch_size = batch_size

        # Its tells us num of times record() was called.
        self.buffer_counter = 0

        # Instead of list of tuples as the exp.replay concept go
        # We use different np.arrays for each tuple element

        if not len(savedBuffers):
            self.state_buffer = np.zeros((self.buffer_capacity, num_states))
            self.action_buffer = np.zeros((self.buffer_capacity, num_actions))
            self.reward_buffer = np.zeros((self.buffer_capacity, 1))
            self.next_state_buffer = np.zeros((self.buffer_capacity, num_states))
            self.done_state_buffer = np.zeros((self.buffer_capacity, 1))
        else:
            self.state_buffer = savedBuffers["states"]
            self.action_buffer = savedBuffers["actions"]
            self.reward_buffer = savedBuffers["rewards"]
            self.next_state_buffer = savedBuffers["nextStates"]
            self.done_state_buffer = savedBuffers["done"]

    def push(self, state, action, reward, next_state, done):
        index = self.buffer_counter % self.buffer_capacity

        self.state_buffer[index] = state
        self.action_buffer[index] = action
        self.reward_buffer[index] = reward
        self.next_state_buffer[index] = next_state
        self.done_state_buffer[index] = done

        self.buffer_counter += 1

    def sample(self):
        record_range = min(self.buffer_counter, self.buffer_capacity)
        # Randomly sample indices
        batch_indices = np.random.choice(record_range,
                                         [self.batch_size if record_range >= self.batch_size else record_range][0],
                                         replace=False)

        # Convert to tensors
        state_batch = self.state_buffer[batch_indices]
        action_batch = self.action_buffer[batch_indices]
        reward_batch = self.reward_buffer[batch_indices]
        next_state_batch = self.next_state_buffer[batch_indices]
        done_state_batch = self.done_state_buffer[batch_indices]
        return (state_batch, action_batch, reward_batch, next_state_batch, done_state_batch)

    def __len__(self):
        return self.buffer_counter
    

class Randnormalnoise(object):
    def __init__(self, action_space, std=0.1):
        
        random.seed(seed)
        
        self.action_space = action_space
        self.std = std
        
    def get_action(self, action):
        nois = np.random.normal(size=self.action_space.shape[0], scale=self.std)
        return np.array(action+nois)


class Critic(nn.Module):
    def __init__(self, input_size, hidden_size, output_size, state_size, name, chkpt_dir='tmp/ddpg'):
        
        torch.manual_seed(seed)            # CPU seed
        torch.cuda.manual_seed(seed)       # current GPU seed
        torch.cuda.manual_seed_all(seed)   # all GPU seed
        
        super(Critic, self).__init__()
        self.critic1_stlinear1 = nn.Linear(state_size, 16)
        self.critic1_stlinear2 = nn.Linear(16, 32)
        self.critic1_aclinear1 = nn.Linear(output_size, 32)
        self.critic1_linear1 = nn.Linear(64, hidden_size)
        self.critic1_linear2 = nn.Linear(hidden_size, hidden_size)
        self.critic1_linear3 = nn.Linear(hidden_size, output_size)

        self.critic2_stlinear1 = nn.Linear(state_size, 16)
        self.critic2_stlinear2 = nn.Linear(16, 32)
        self.critic2_aclinear1 = nn.Linear(output_size, 32)
        self.critic2_linear1 = nn.Linear(64, hidden_size)
        self.critic2_linear2 = nn.Linear(hidden_size, hidden_size)
        self.critic2_linear3 = nn.Linear(hidden_size, output_size)
        
        self.checkpoint_dir = chkpt_dir
        self.checkpoint_file = os.path.join(self.checkpoint_dir, name+'_ddpg')

    def forward(self, state, action):
        """
        Params state and actions are torch tensors
        """
        state_x = F.relu(self.critic1_stlinear1(state))
        state_x = F.relu(self.critic1_stlinear2(state_x))
        action_x = F.relu(self.critic1_aclinear1(action))
        q1 = torch.cat([state_x, action_x], 1)
        q1 = F.relu(self.critic1_linear1(q1))
        q1 = F.relu(self.critic1_linear2(q1))
        q1 = self.critic1_linear3(q1)

        state_x = F.relu(self.critic2_stlinear1(state))
        state_x = F.relu(self.critic2_stlinear2(state_x))
        action_x = F.relu(self.critic2_aclinear1(action))
        q2 = torch.cat([state_x, action_x], 1)
        q2 = F.relu(self.critic2_linear1(q2))
        q2 = F.relu(self.critic2_linear2(q2))
        q2 = self.critic2_linear3(q2)
        return q1, q2

    def critic_1(self, state, action):
        state_x = F.relu(self.critic1_stlinear1(state))
        state_x = F.relu(self.critic1_stlinear2(state_x))
        action_x = F.relu(self.critic1_aclinear1(action))
        q1 = torch.cat([state_x, action_x], 1)
        q1 = F.relu(self.critic1_linear1(q1))
        q1 = F.relu(self.critic1_linear2(q1))
        q1 = self.critic1_linear3(q1)
        return q1

    def save_checkpoint(self):
        #print('... saving checkpoint ...')
        T.save(self.state_dict(), self.checkpoint_file)

    def load_checkpoint(self):
        # print('... loading checkpoint ...')
        self.load_state_dict(T.load(self.checkpoint_file))


class Actor(nn.Module):
    def __init__(self, input_size, hidden_size, output_size, name, learning_rate=3e-4, chkpt_dir='tmp/ddpg'):
        
        torch.manual_seed(seed)            # CPU seed
        torch.cuda.manual_seed(seed)       # current GPU seed
        torch.cuda.manual_seed_all(seed)   # all GPU seed
        
        super(Actor, self).__init__()
        self.linear1 = nn.Linear(input_size, hidden_size)
        self.linear2 = nn.Linear(hidden_size, hidden_size)
        self.linear3 = nn.Linear(hidden_size, output_size)

        self.checkpoint_dir = chkpt_dir
        self.checkpoint_file = os.path.join(self.checkpoint_dir, name+'_ddpg')

    def forward(self, state):
        """
        Param state is a torch tensor
        """
        x = F.relu(self.linear1(state))
        x = F.relu(self.linear2(x))
        x = torch.tanh(self.linear3(x))
        x = x * 2
        return x

    def save_checkpoint(self):
        #print('... saving checkpoint ...')
        T.save(self.state_dict(), self.checkpoint_file)

    def load_checkpoint(self):
        # print('... loading checkpoint ...')
        self.load_state_dict(T.load(self.checkpoint_file))


# %%
class TD3agent:
    def __init__(self, env, droprob, l2penalty, batch_size, policy_freq, hidden_size=64, actor_learning_rate=1e-3, critic_learning_rate=1e-3, gamma=0.99,
                 tau=1e-3, max_memory_size=int(50000), policy_noise=0.1, noise_clip=0.5):
        
        torch.manual_seed(seed)            # CPU seed
        torch.cuda.manual_seed(seed)       # current GPU seed
        torch.cuda.manual_seed_all(seed)   # all GPU seed
        
        # Params
        self.num_states = env.observation_space.shape[0]
        self.num_actions = env.action_space.shape[0]
        self.gamma = gamma
        self.tau = tau
        self.policyNoise = policy_noise
        self.noiseClip = noise_clip
        self.policyFreq = policy_freq
        self.policyIterationCount = 0
        self.droprob = droprob
        self.l2penalty = l2penalty

        # Networks
        self.actor = Actor(self.num_states, hidden_size, self.num_actions, name='actor')
        self.actor_target = Actor(self.num_states, hidden_size, self.num_actions, name='actor_target')
        self.critic = Critic(self.num_states + self.num_actions, hidden_size, self.num_actions, self.num_states, name='critic')
        self.critic_target = Critic(self.num_states + self.num_actions, hidden_size, self.num_actions, self.num_states, name='critic_target')
        
        for target_param, param in zip(self.actor_target.parameters(), self.actor.parameters()):
            target_param.data.copy_(param.data)

        for target_param, param in zip(self.critic_target.parameters(), self.critic.parameters()):
            target_param.data.copy_(param.data)

        # Training
        self.memory = Memory(max_memory_size, batch_size, self.num_states, self.num_actions)
        self.critic_criterion = nn.MSELoss()
        self.actor_optimizer = optim.Adam(self.actor.parameters(), lr=actor_learning_rate, weight_decay=self.l2penalty)
        self.critic_optimizer = optim.Adam(self.critic.parameters(), lr=critic_learning_rate, weight_decay=self.l2penalty)

    def get_action(self, state):
        state = Variable(torch.from_numpy(np.array(state)).float().unsqueeze(0))
        action = self.actor.forward(state)
        action = action.detach().numpy().flatten()
        return action

    def update(self):
        self.policyIterationCount += 1
        states, actions, rewards, next_states, done_ = self.memory.sample()
        states = torch.FloatTensor(states)
        actions = torch.FloatTensor(actions)
        rewards = torch.FloatTensor(rewards)
        next_states = torch.FloatTensor(next_states)
        doneState = torch.FloatTensor(done_)

        # we don't call backward or apply backpropagation for target networks, so we can use no_grad to speed up the
        # computation
        with torch.no_grad():
            # Select action according to policy and add clipped noise
            noise = (
                    torch.randn_like(actions) * self.policyNoise
            ).clamp(-self.noiseClip, self.noiseClip)

            next_action = (
                    self.actor_target.forward(next_states) + noise
            ).clamp(env.action_space.low[0], env.action_space.high[0])

            # Compute the target Q value
            target_Q1, target_Q2 = self.critic_target.forward(next_states, next_action)
            target_Q = torch.min(target_Q1, target_Q2)
            target_Q = rewards + self.gamma * target_Q * (1 - doneState)

        # Get current Q estimates
        current_Q1, current_Q2 = self.critic.forward(states, actions)

        # Compute critic loss
        critic_loss = F.mse_loss(current_Q1, target_Q) + F.mse_loss(current_Q2, target_Q) #可以相加,因为另一个网络的loss相当于常数项,对本身网络参数求梯度后为0

        # Optimize the critic
        self.critic_optimizer.zero_grad()
        critic_loss.backward()
        self.critic_optimizer.step()

        # Delayed policy updates
        if self.policyIterationCount % self.policyFreq == 0:

            # Compute actor losse
            actor_loss = -self.critic.critic_1(states, self.actor(states)).mean()

            # Optimize the actor
            self.actor_optimizer.zero_grad()
            actor_loss.backward()
            self.actor_optimizer.step()

            # Update the frozen target models
            for target_param, param in zip(self.actor_target.parameters(), self.actor.parameters()):
                target_param.data.copy_(param.data * self.tau + target_param.data * (1.0 - self.tau))

            for target_param, param in zip(self.critic_target.parameters(), self.critic.parameters()):
                target_param.data.copy_(param.data * self.tau + target_param.data * (1.0 - self.tau))

    def clip_grad_norm_(self, module, max_grad_norm):
        nn.utils.clip_grad_norm_([p for g in module.param_groups for p in g["params"]], max_grad_norm)

    def save_models(self):
        self.actor.save_checkpoint()
        self.actor_target.save_checkpoint()
        self.critic.save_checkpoint()
        self.critic_target.save_checkpoint()
    
    def load_models(self):
        self.actor.load_checkpoint()
        self.actor_target.load_checkpoint()
        self.critic.load_checkpoint()
        self.critic_target.load_checkpoint() 



#%%
if __name__ == "__main__":
    np.random.seed(seed)
    T.manual_seed(seed)
    env = frmwrk.SimulationEnvironment()
    
    batch_size = 32 #1; 2; 4; 8; 16; 32; 64; 128; 256
    droprob = 0 #0; 0.1; 0.2; 0.3; 0.4; 0.5
    l2penalty = 0 #0; 0.05; 0.1; 0.15; 0.2; 0.25; 0.3; 0.35; 0.4; 0.45; 0.5
    policy_freq = 2 #2; 4; 8
    
    agent = TD3agent(env, droprob, l2penalty, batch_size, policy_freq)
    rdnoise = Randnormalnoise(env.action_space)
    
    msg = []
    tot_info = []
    
    num_episodes = 200
    num_steps = 50 # n_steps*0.5s
    best_score = float('-inf')
    
    ep_rewards = []
    
    for episode in tqdm(range(num_episodes)):
        state = env.reset()
        episode_reward = 0

        for step in range(num_steps):
            action = agent.get_action(state)
            action = rdnoise.get_action(action)
            action = np.clip(action, env.action_space.low[0], env.action_space.high[0])
            try:
                new_state, reward, done, info = env.startNextStep(action)
            except Exception as e:
                pass
            agent.memory.push(state, action, reward, new_state, done)
            
            if episode == num_episodes - 1:
                tot_info.append(info)
            
            # if len(agent.memory) > batch_size:
            agent.update()
            
            state = new_state
            episode_reward += reward

            
            if done and env.zeroVelCounter >= 50:
                print("ending episode due to zero velocity")
                break
    
            elif done:
                print('Collision might happened, end game')
                break

        ep_rewards.append(episode_reward)
        avg_score = np.mean(ep_rewards[-50:])
        if avg_score > best_score:
            best_score = avg_score
            agent.save_models()
            print('model saved')
        
        msg.append('episode {}, score {}, average score {}'.format(episode, round(episode_reward,4), round(avg_score,4)))
        print('episode {}, score {}, average score {}'.format(episode, round(episode_reward,4), round(avg_score,4)))
    
    env.stopSimulation("stop")
    
#%%

    tot_info = np.array(tot_info, dtype = object)
    np.save('msg_{}_TD3_dp{}_wd{}_pf{}.npy'.format(batch_size, str(droprob).replace('.', 'x'), str(l2penalty).replace('.', 'x'), policy_freq), np.array(msg))
    # np.save('tot_info.npy', tot_info)

